classdef DesignBoat <Design
    %D=DesignBoat creates an instance of a design D(x) given by:
    %
    %       D(x)={ y in R^2 with:
    %               (y1-x1)^2 + (y2-x2)^2 - x3^2<= 0
    %               (y1-[x1+x3*x4/sqrt(x4^2+x5^2)])^2 + (y2-[x2+x3*x5/sqrt(x4^2+x5^2)])^2 - x3^2 <= 0}
    %       
    %       Vol(x)=(pi/3- sqrt(3)/2)*x3^2
    %
    %       x in X = { x in R^5 with
    %               10^-6 <= x3
    %               -x4^2-x5^2+1 <=0}

    
    properties (GetAccess=protected)
        
        dim=2;                                  
        nrOfParams=5;                           
        nrOfDesignConstrs=2;                     
        initValsParams=[0,0,1,1,0];         
        
        linEqParamConstrs=[] ;                   
        linIneqParamConstrs=[];  
        boundsOnParams=[-inf,-inf,1e-6,-inf,-inf;inf,inf,inf,inf,inf,]
        
        nrOfNonlinParamEqConstrs=0;              
        nrOfNonlinParamIneqConstrs=1;            

       
    end
    
    methods
        function v=vol(~,x)                     
            v=(pi/3-sqrt(3)/2)*x(3)^2;
        end
        function grad_v=gradVol(~,x)           
            grad_v=[0,0,2*(pi/3-sqrt(3)/2)*x(3),0,0]';
        end
        
        %design functions
        function u=evalNonlinDescrFuns(~,x,y)                
            y1=y(:,1);
            y2=y(:,2);
    
            o = ones(size(y1));
        
            u = zeros(length(y1),2);
            u(:,1) = (y1 - x(1)*o).^2 + (y2 - x(2)*o).^2 - x(3)^2*o;
            l = sqrt(x(4)^2 + x(5)^2);
            u(:,2) = (y1 - (x(1) + x(3)*x(4)/l)*o).^2 + (y2 - (x(2) + x(3)*x(5)/l)*o).^2 - x(3)^2*o;
        end    
        function Dx_u=DxNonlinDescrFuns(~,x,y)               
            
              Dx_u = zeros(2,5);

              Dx_u(1,1) = 2*(x(1) - y(1));
              Dx_u(1,2) = 2*(x(2) - y(2));
              Dx_u(1,3) = -2*x(3);

              t1 = x(4) ^ 2;
              t2 = x(5) ^ 2;
              t3 = t1 + t2;
              t4 = sqrt(t3);
              t7 = x(3) * x(4);
              t9 = 0.1e1 / t4;
              t14 = x(3) * x(5);
              t32 = (-x(5) * y(1) + x(5) * x(1) + x(4) * y(2) - x(2) * x(4)) / t4 / t3;
              Dx_u(2,1) = 0.2e1 * (-y(1) * t4 + x(1) * t4 + t7) * t9;
              Dx_u(2,2) = 0.2e1 * (-y(2) * t4 + x(2) * t4 + t14) * t9;
              Dx_u(2,3) = 0.2e1 * t9 * (-y(1) * x(4) + x(4) * x(1) - y(2) * x(5) + x(5) * x(2));
              Dx_u(2,4) = 0.2e1 * t14 * t32;
              Dx_u(2,5) = -0.2e1 * t7 * t32;
        end
        function Dy_u=DyNonlinDescrFuns(~,x,y)
            Dy_u = zeros(2,2);
            Dy_u(1,1) = 2*(y(1) - x(1));
            Dy_u(1,2) = 2*(y(2) - x(2));
            t2 = x(4)^2;
            t3 = x(5)^2;
            t5 = sqrt(t2 + t3);
            t6 = 0.1e1 / t5;
            Dy_u(2,1) = 2*(y(1) - x(1)) - 0.2e1 * x(3) * x(4) * t6;
            Dy_u(2,2) = 2*(y(2) - x(2)) - 0.2e1 * x(3) * x(5) * t6;
        end
        
        function y=getInteriorPoint(~,x)                    
            y = x(1:2)+0.25*x(3)*x(4:5);
        end
        function Dx_y=DxInnerPoint(~,x)
            Dx_y=[eye(2),0.25*x(4:5)',0.25*x(3)*eye(2)];
        end
        
        function [pt_min, pt_max] = getBoundingBox(obj,x) 

            pt_min=[x(1)-x(3),x(2)-x(3)];
            pt_max=[x(1)+x(3),x(2)+x(3)];
            %exact bounds
%           assert(length(x) == 5);
% 
%           myFminconOptions = optimset('Algorithm', 'sqp');
%           myFminconOptions = optimset(myFminconOptions, 'Display', 'off');
%           myFminconOptions = optimset(myFminconOptions, 'MaxFunEvals', Inf);
%           myFminconOptions = optimset(myFminconOptions, 'FunValCheck', 'on');
%           myFminconOptions = optimset(myFminconOptions, 'GradObj', 'on', 'GradConstr', 'on', 'DerivativeCheck', 'off');
%           myFminconOptions = optimset(myFminconOptions, 'ScaleProblem', 'obj-and-constr');
%           myFminconOptions = optimset(myFminconOptions, 'FinDiffType', 'central');
% 
%           y0 = obj.getInteriorPoint(x);
%           %obj.evalNonlinDescrFuns(x,y0)
%           pt_y1_min = fmincon(@ObjFun, y0, [], [], [], [], [], [], @NonlinConFuns, myFminconOptions,obj, 1, 1, x);
%           %obj.evalNonlinDescrFuns(x,pt_y1_min)
%           pt_y1_max = fmincon(@ObjFun, y0, [], [], [], [], [], [], @NonlinConFuns, myFminconOptions,obj, 1, -1, x);
%           %obj.evalNonlinDescrFuns(x,pt_y1_max)
%           pt_y2_min = fmincon(@ObjFun, y0, [], [], [], [], [], [], @NonlinConFuns, myFminconOptions,obj, 2, 1, x);
%           %obj.evalNonlinDescrFuns(x,pt_y2_min)
%           pt_y2_max = fmincon(@ObjFun, y0, [], [], [], [], [], [], @NonlinConFuns, myFminconOptions,obj, 2, -1, x);
%           %obj.evalNonlinDescrFuns(x,pt_y2_max)
%           pt_min = [pt_y1_min(1), pt_y2_min(2)];
%           pt_max = [pt_y1_max(1), pt_y2_max(2)];
        function [f, grad_f] = ObjFun(y,~,ID,sig,~)

          f = sig*y(ID);

          if (nargout > 1)

            grad_f = zeros(2,1);
            grad_f(ID) = sig;

          end

        end
%         function [Dx_lb,Dx_ub]=DxBoundingBox(obj,x)
%             
%         end
        
        function [g, h, grad_g, grad_h] = NonlinConFuns(y,obj1,~,~,x)

          g = obj1.evalNonlinDescrFuns(x,y);
          h = [];

          if (nargout > 2)

            grad_g = obj1.DyNonlinDescrFuns(x,y);
            grad_h = [];

          end

        end

        end
        
        %non-linear constraints on parameters
        function k=evalNonlinIneqConstrsOnParams(~,x)        
            k=-x(4)^2-x(5)^2+1;
        end    
        function Dx_k=DxNonlinIneqConstrsOnParams(~,x)       
            
            Dx_k=[0,0,0,-2*x(4),-2*x(5)];
        end    
        

    end
    
    %transformation    
    methods
        function y=evalTrafoFun(~,x,z)
            y = zeros(size(z));
  
            IDsleq = find(z(:,2) <= 0);
            IDsg = find(z(:,2) > 0);
  
            o1 = ones(length(IDsleq),1);
            o2 = ones(length(IDsg),1);
  
            y(IDsleq,1) = x(1)*o1 + x(3)*z(IDsleq,1).*cos(atan2(x(5), x(4))*o1 + pi / 0.3e1 *o1 + 0.2e1 / 0.3e1 * pi * z(IDsleq,2));
            y(IDsleq,2) = x(2)*o1 + x(3)*z(IDsleq,1).*sin(atan2(x(5), x(4))*o1 + pi / 0.3e1 *o1 + 0.2e1 / 0.3e1 * pi * z(IDsleq,2));

            y(IDsg,1) = x(1)*o2 + x(3) * x(4) * (x(4) ^ 2 + x(5) ^ 2) ^ (-0.1e1 / 0.2e1) * o2 + ...
                      x(3) * z(IDsg,1) .* cos(atan2(x(5), x(4))*o2 - 0.4e1 / 0.3e1 * pi*o2 + 0.2e1 / 0.3e1 * pi * z(IDsg,2));
            y(IDsg,2) = x(2)*o2 + x(3) * x(5) * (x(4) ^ 2 + x(5) ^ 2) ^ (-0.1e1 / 0.2e1) * o2 + ...
                      x(3) * z(IDsg,1) .* sin(atan2(x(5), x(4))*o2 - 0.4e1 / 0.3e1 * pi*o2 + 0.2e1 / 0.3e1 * pi * z(IDsg,2));

        end
        function z=evalInvTrafoFun(~,x,y)
            m=size(y,1);
            o = ones(size(y,1),1);
  
            l = norm(x(4:5));
            n0 = 1/l*x(4:5)';
            c = x(1:2) + 0.5*x(3)*n0';

            theta1 = atan2(x(5),x(4));
            theta2 = theta1-pi;

            IDsleq = find((y(:,1)-c(1)*o)*n0(1) + (y(:,2)-c(2)*o)*n0(2) <= 0);
            
            IDsg =find((y(:,1)-c(1)*o)*n0(1) + (y(:,2)-c(2)*o)*n0(2) > 0);

            o1 = ones(length(IDsleq),1);
            o2 = ones(length(IDsg),1);

            y1t = (y(:,1)-x(1)*o)/x(3);
            y2t = (y(:,2)-x(2)*o)/x(3);

            if ~isempty(IDsleq)
              z(IDsleq,1) = sqrt((y1t(IDsleq)-x(4)/l*o1).^2 + (y2t(IDsleq)-x(5)/l*o1).^2);
              z(IDsleq,2) = 3/(2*pi)*(atan2(y2t(IDsleq)-x(5)/l*o1,y1t(IDsleq)-x(4)/l*o1) - (theta2-pi/3)*o1);
            end

            if ~isempty(IDsg)
              z(IDsg,1) = sqrt((y1t(IDsg)).^2 + (y2t(IDsg)).^2);
              z(IDsg,2) = 3/(2*pi)*(atan2(y2t(IDsg),y1t(IDsg)) - (theta1+pi/3)*o2);
            end
        end
        function Dx_y=DxTrafoFun(obj,x,z)
             m = size(z,1);
            Dx_y = zeros(2,5,m);
            myEps = 1.e-6;
            for i = 1:5

                x_1 = x;
                x_2 = x;

                x_1(i) = x_1(i)-myEps;
                x_2(i) = x_2(i)+myEps;

                Y_1 = obj.evalTrafoFun(x_1, z);
                Y_2 = obj.evalTrafoFun(x_2, z);

                Dx_y(:,i,:) = 1/(2*myEps)*(Y_2'-Y_1');
            end
        end
    end
    
       methods
        function []=plotDesign(obj,x)
            t = (0:pi/180:pi);
            o = ones(size(t));
            ang=atan2(x(5), x(4));

            y1_1 = x(1) + x(3)*cos(ang + pi / 0.3e1  + 0.2e1 / 0.3e1 *(t-pi)) ;
            y2_1 = x(2) + x(3)*sin(ang + pi / 0.3e1  + 0.2e1 / 0.3e1 *(t-pi));

            y1_2 = x(1) + x(3) * x(4) * (x(4) ^ 2 + x(5) ^ 2) ^ (-0.1e1 / 0.2e1)  + x(3)*cos(ang - 0.4e1 / 0.3e1 * pi + 0.2e1 / 0.3e1*t);
            y2_2 = x(2) + x(3) * x(5) * (x(4) ^ 2 + x(5) ^ 2) ^ (-0.1e1 / 0.2e1)  + x(3)*sin(ang - 0.4e1 / 0.3e1 * pi + 0.2e1 / 0.3e1*t);

            y1=[y1_1,y1_2];
            y2=[y2_1,y2_2];
            

            fill(y1,y2,'green');
            hold on;            
        end
    end
     
    
end

