classdef DesignHyperBall <Design
    % D=DesignHyperBall(n) creates an instance of an n-dimensional design D(x) 
    %  given by:
    %
    %       D(x)={ y in R^n with:
    %              sum_i=1^n(yi-xi)^2 -xn+1^2<= 0}
    %       
    %       Vol(x)=V*xn+1^2
    %           V is the volume for xn+1 = 1
    %
    %       x in X = { x in R^(n+1) with
    %               10^-6 <= xn+1}
    
    
    properties (GetAccess=protected)
        dim                                     
        nrOfParams                              
        nrOfDesignConstrs  =1;                   
        initValsParams 
        BoundsOnParams
        
        linEqParamConstrs =[];                   
        linIneqParamConstrs=[];                     
        boundsOnParams
        
        nrOfNonlinParamEqConstrs =0;             
        nrOfNonlinParamIneqConstrs  =0;
        
        %Volume of the unit-sphere
        volsphere;
       
    end
    
    methods
        function p=DesignHyperBall(dim,varargin)
            
            %compute properties with dim
            p.dim=dim;
            p.nrOfParams=dim+1;
            p.initValsParams=[zeros(1,dim),1];
            p.boundsOnParams=[-inf*ones(1,dim),1.e-6;inf*ones(1,dim+1)];
            %volume of the unit-sphere
            if mod(dim,2)
                k=(dim-1)/2;
                p.volsphere=2*(2*pi)^k/prod(1:2:dim);
            else
                k=dim/2;
                p.volsphere=pi^k/factorial(k);
            end
            
%             %Erstellen der Plotrutinen
%             if dim == 2
%                 p.canPlot='on';
%                 p.designPlot=designPlotCircle;
%             elseif dim ==3
%                 p.canPlot='on';
%                 p.designPlot=designPlotHyperBall3D;
%             else
%                 p.canPlot='off';
%             end
%             
%             if strcmp(p.trans.getName,'transHyperBall') && p.dim==2
%                 p.Circle=true;
%             end
            
        end
    end
    
    
    methods
        
        
        %volume
        function v=vol(obj,x)                      
            v=obj.volsphere*x(end)^obj.dim;
        end
        function grad_v=gradVol(obj,x)           
            grad_v=zeros(obj.nrOfParams,1);
            grad_v(end)=obj.volsphere*(obj.dim)*x(end)^(obj.dim-1);
        end
        
        %design functions
        function u=evalNonlinDescrFuns(~,x,y)                
            u=sum((y-x(1:end-1)).^2)-x(end)^2;
           
        end    
        function Dy_u=DyNonlinDescrFuns(~,x,y)               
            Dy_u= 2*(y-x(1:end-1));  
        end
        function Dx_u=DxNonlinDescrFuns(~,x,y)
            Dx_u= [-2*(y-x(1:end-1)),-2*x(end)];
        end
        %interior point
        function y=getInteriorPoint(~,x)                    
            y=x(1:end-1);
            
        end
        function Dx_y=DxInnerPoint(obj,~)
            Dx_y=[eye(obj.dim),zeros(obj.dim,1)];
        end
        %bounding box
        function [lb,ub]=getBoundingBox(obj,x)          
            n=obj.dim;
            lb=x(1:n)-x(end);
            ub=x(1:n)+x(end);
        end
        function [Dx_lb,Dx_ub]=DxBoundingBox(obj,x)          
            n=obj.dim;
            Dx_lb=[eye(n),-ones(n,1)];
            Dx_ub=[eye(n),ones(n,1)];
        end
        
        %Transformation Function
        function y=evalTrafoFun(obj,x,z)
            m=size(z,1);
            y=x(end)*z+ones(m,1)*x(1:end-1);
        end
        
        function z=evalInvTrafoFun(obj,x,y)
            m=size(y,1);
            temp=y-ones(m,1)*x(1:end-1);
            z=temp/x(end);
            
            
        end
        
        function Dx_y=DxTrafoFun(obj,x,z)
            m=size(z,1);
            Dx_y=zeros(obj.dim,obj.nrOfParams,m);
            for i=1:m
                Dx_y(:,:,i)=[eye(obj.dim),z(i,:)'];
            end

            
        end
        %plot-function
        function plotDesign(obj,x)
            switch obj.dim
                case 2
                    t = 0:pi/180:2*pi;
                    o = ones(size(t));

                    y1 = x(1)*o+x(3)*cos(t);
                    y2 = x(2)*o+x(3)*sin(t);

                    fill(y1,y2,'green');
                    hold on;
                case 3
                   [y1t,y2t,y3t]=sphere();
                    y1=x(4)*y1t+x(1);
                    y2=x(4)*y2t+x(2);
                    y3=x(4)*y3t+x(3);
                    s = surfl(y1, y2, y3);
                    set(s, 'EdgeColor', 'none' , 'FaceColor', 'green', 'FaceAlpha', 0.75, 'FaceLighting', 'gouraud');
                    camlight; 
                otherwise
                    warning('Can not plot the problem in dimension higher than 3')
            end
        end
        
    end
    
        
    
end




