classdef Problem22 < ImplicitGSIPProblem
    % p=Problem22 creates an instance of the GSIP problem:
    %
    %       min   x1^2+x2^2+x3^2
    %
    %       s.t.  x1(y1+y2^2+1)+x2(y1y2-y2^2)+x3(y1y2+y2^2+y2) +12 <=0 for all y in Y(x)
    %             -1 <= x1 <= 0
    %             -1 <= x2 <= 0
    %             -1 <= x3 <= 0    
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               x1^2-y1^2<=0 
    %               0 <= y1 <= 1
    %               0 <= y2 <= 1}
    
    properties(GetAccess=protected)
        
        nrOfUppLevVars=3;                         
        boundsOnUppLevVars=[-1,-1,-1;0,0,0];       
        nrOfSemiInfConstrs=1;                        
        initValsOfUppLevVars=[-0.5,-0.5,-0.5];      
        
        linEqsOnUppLevVars=[];                     
        linIneqsOnUppLevVars=[];                   
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=0;              
        
        
        nrOfLowLevVars=2;                          
        nrOfLowLevNonlinConstrs=1;                        
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end

    
    methods
        %objective function
        function f=evalObjFun(obj,x)                    
            f=x(1)^2+x(2)^2+x(3)^2;
        end
        function grad_f=gradObjFun(obj,x)                   
            grad_f=2*x';
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,i,x,y)      
                                                         
            g=x(1)*(y(:,1)+y(:,2).^2+1)+x(2)*(y(:,1).*y(:,2)-y(:,2).^2)+x(3)*(y(:,1).*y(:,2)+y(:,2).^2+y(:,2))+12;            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)     

            Dx_g=[y(:,1)+y(:,2).^2+1,y(:,1).*y(:,2)-y(:,2).^2,y(:,1).*y(:,2)+y(:,2).^2+y(:,2)];                
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)
            Dy_g=[x(1)+x(2)*y(:,2)+x(3)*y(:,2),...
                2*x(1)*y(:,2)+x(2)*(y(:,1)-2*y(:,2))+x(3)*(y(:,1)+2*y(:,2)+1)];                
        end
                      
        %lower level
        function u=evalLowLevNonlinConstrs(obj,~,x,y)           
            u=[x(1)^2-y(1)^2];                        
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,~,x,y)              
            Dy_u=[-2*y(1),0];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,~,x,y)              
            Dx_u=[2*x(1),0,0];
        end
            
        function y=getInitValsLowLevVars(obj,~,x)         
            [lb,ub] = obj.getBox(x);
            y=(lb+ub)/2;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            [Dx_lb,Dx_ub]=DxInterval(x);
            Dx_y=(Dx_lb+Dx_ub)/2;
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,x)    
             lb=[0,0];
             ub=[1,1];
        end
                        

        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getBox(x);
            n=size(z,1);
             y=ones(n,1)*lb+z*diag((ub-lb));            
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            n=size(y,1);              
            [lb,ub]=obj.getBox(x);
            z=(y-ones(n,1)*lb)*diag(1./((ub-lb)));
            
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            n=size(z,1);
            [Dx_lb,Dx_ub]=obj.DxBox(x);
            Dx_y=zeros(size(z,2),size(x,2),n);
            for i=1:n
                Dx_y(:,:,i)=Dx_lb+diag(z(i,:))*(Dx_ub-Dx_lb);
            end
        end

    end
        methods(Access=private)
        function [lb,ub] = getBox(obj,x)           
            lb=[sqrt(x(2)^2),0];
            ub=[1,1];
        end
        function [Dx_lb,Dx_ub]=DxBox(obj,x)
            if x(2)==0
               Dx_lb=[0,0,0;0,0,0]; 
            else
               Dx_lb=[0,x(2)/abs(x(2)),0;0,0,0];  
            end
            
            Dx_ub=[0,0,0;0,0,0];
        end
    end
    
end
